package net.customware.confluence.reporting.macro;

import com.atlassian.confluence.core.ContentEntityObject;
import com.atlassian.renderer.v2.RenderMode;
import com.atlassian.renderer.v2.RenderUtils;
import com.atlassian.renderer.v2.macro.MacroException;
import net.customware.confluence.reporting.ReportBuilder;
import net.customware.confluence.reporting.ReportException;
import org.apache.log4j.Logger;
import org.randombits.confluence.support.ConfluenceMacro;
import org.randombits.confluence.support.MacroInfo;
import org.randombits.confluence.support.ServletAssistant;

import javax.servlet.http.HttpServletRequest;

/**
 * this is the abstract base class for all reporting macros. It simply makes it
 * easier to handle the many {@link ReportException}s generated by reporting
 * method calls.
 * 
 * Subclasses should implement the {@link #report(MacroInfo)} method in the same way
 * you would usually implement the {@link #execute(MacroInfo)} method, but throw
 * the extra {@link ReportException}.
 */
public abstract class AbstractReportingMacro extends ConfluenceMacro {

    private static final Logger LOG = Logger.getLogger( AbstractReportingMacro.class );


    @Override protected final String execute( MacroInfo info ) throws MacroException {
        try {
            return report( info );
        } catch ( ReportException e ) {
            String path = null;
            HttpServletRequest request = ServletAssistant.getInstance().getRequest();
            if ( request != null ) {
                path = request.getRequestURI() + (request.getQueryString() != null ? "?" + request.getQueryString() : "" ) ;
            }
            if ( path == null ) {
                ContentEntityObject content;
                try {
                    content = ReportBuilder.getRootContent();
                } catch ( ReportException e1 ) {
                    content = info.getContent();
                }

                if (content != null)
                    path = content.getUrlPath();
            }

            String onPath = path == null ? "" : " on \"" + path + "\"";

            LOG.info( "Error while reporting" + onPath + ": " + e.getMessage(), e );
            return RenderUtils.blockError( "", e.getMessage() );
        }
    }

    protected abstract String report( MacroInfo info ) throws MacroException, ReportException;
    
    protected String renderWiki( String wiki, RenderMode renderMode ) throws ReportException {
        return ReportBuilder.renderWiki( wiki, renderMode, this );
    }
}
